"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fontWidth_1 = require("./fontWidth");
function fitColumnWidths(grid, renderer, options = {}) {
    const estimator = new ColumnWidthEstimator(grid.model, renderer, options);
    const widths = estimator.getColumnWidths();
    widths.forEach((width, column) => {
        grid.resizeSection('column', column, width);
    });
    const headerWidth = estimator.getRowHeaderWidth();
    grid.resizeSection('row-header', 0, headerWidth);
}
exports.fitColumnWidths = fitColumnWidths;
var FitColumnWidths;
(function (FitColumnWidths) {
    FitColumnWidths.defaultRowsToInspect = 100;
    FitColumnWidths.defaultMinWidth = 40;
    FitColumnWidths.defaultMaxWidth = 300;
    FitColumnWidths.characterScaleFactor = 0.65;
})(FitColumnWidths = exports.FitColumnWidths || (exports.FitColumnWidths = {}));
class ColumnWidthEstimator {
    constructor(model, renderer, options = {}) {
        this._model = model;
        this._renderer = renderer;
        this._rowsToInspect =
            options.rowsToInspect || FitColumnWidths.defaultRowsToInspect;
        this._minWidth = options.minWidth || FitColumnWidths.defaultMinWidth;
        this._maxWidth = options.maxWidth || FitColumnWidths.defaultMaxWidth;
        this._characterScaleFactor =
            options.characterScaleFactor || FitColumnWidths.characterScaleFactor;
    }
    getColumnWidths() {
        const numberColumns = this._model.columnCount('body');
        const widths = Array.from({ length: numberColumns }, (_, column) => this._getColumnWidth(column));
        return widths;
    }
    getRowHeaderWidth() {
        if (this._model.columnCount('row-header') !== 1) {
            throw new Error('Unsupported grid: row header does not contain exactly one column');
        }
        const headerData = this._getDataFromRegion('corner-header', 0);
        const bodyData = this._getDataFromRegion('row-header', 0);
        const width = this._measureArrayWidth([...headerData, ...bodyData]);
        return this._clampWidth(width);
    }
    _getColumnWidth(column) {
        const headerData = this._getDataFromRegion('column-header', column);
        const bodyData = this._getDataFromRegion('body', column);
        const width = this._measureArrayWidth([...headerData, ...bodyData]);
        return this._clampWidth(width);
    }
    _getDataFromRegion(region, column) {
        const rowsToCheck = Math.min(this._getRowCount(region), this._rowsToInspect);
        const data = Array.from({ length: rowsToCheck }, (_, idx) => this._model.data(region, idx, column));
        return data;
    }
    _measureArrayWidth(content) {
        const widths = content.map(elementContent => this._measureElementWidth(elementContent));
        if (widths.length === 0) {
            return 0;
        }
        else {
            return Math.max(...widths);
        }
    }
    _measureElementWidth(content) {
        const config = {
            x: 1,
            y: 1,
            height: 100,
            width: 20,
            region: 'body',
            row: 10,
            column: 10,
            metadata: {},
            value: content
        };
        const rendered = this._renderer.format(config);
        const width = fontWidth_1.getFontWidth('12px sans-serif');
        return rendered.length * width * this._characterScaleFactor;
    }
    _getRowCount(region) {
        if (region === 'corner-header' || region === 'column-header') {
            return this._model.rowCount('column-header');
        }
        else if (region === 'body' || region === 'row-header') {
            return this._model.rowCount('body');
        }
        else {
            throw 'unreachable';
        }
    }
    _clampWidth(width) {
        return Math.max(Math.min(width, this._maxWidth), this._minWidth);
    }
}
