"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const widgets_1 = require("@phosphor/widgets");
const signaling_1 = require("@phosphor/signaling");
const components_1 = require("./components");
const query_1 = require("./query");
const connection_1 = require("./connection");
const databaseSummary_1 = require("./databaseSummary");
const tableSummary_1 = require("./tableSummary");
const page_1 = require("./page");
class JupyterLabSqlWidget extends widgets_1.Widget {
    constructor(editorFactory, options) {
        super();
        this._toolbar = null;
        this._sqlStatement = '';
        this._page = null;
        this._pageChanged = new signaling_1.Signal(this);
        this._connectionUrlChanged = new signaling_1.Signal(this);
        this._tableNameChanged = new signaling_1.Signal(this);
        this._sqlStatementChanged = new signaling_1.Signal(this);
        this.addClass('jp-MainAreaWidget');
        this.id = 'jupyterlab-sql';
        this._configureTitle();
        this.content = new components_1.SingletonPanel();
        this.layout = this._createWidgetLayout();
        this.name = options.name;
        this.pageName = options.pageName;
        this._connectionUrl = options.connectionUrl;
        this._tableName = options.tableName;
        this._sqlStatement = options.sqlStatement;
        this.editorFactory = editorFactory;
        this._setInitialPage();
    }
    get connectionUrl() {
        return this._connectionUrl;
    }
    get tableName() {
        return this._tableName;
    }
    get sqlStatement() {
        return this._sqlStatement;
    }
    get pageChanged() {
        return this._pageChanged;
    }
    get connectionUrlChanged() {
        return this._connectionUrlChanged;
    }
    get tableNameChanged() {
        return this._tableNameChanged;
    }
    get sqlStatementChanged() {
        return this._sqlStatementChanged;
    }
    onActivateRequest() {
        this._focusContent();
    }
    onCloseRequest() {
        this.dispose();
    }
    _createWidgetLayout() {
        const layout = new widgets_1.BoxLayout({ spacing: 0, direction: 'top-to-bottom' });
        widgets_1.BoxLayout.setStretch(this.content, 1);
        layout.addWidget(this.content);
        this.content.node.tabIndex = -1;
        return layout;
    }
    _configureTitle() {
        this.title.label = 'SQL';
        this.title.closable = true;
    }
    _setInitialPage() {
        if (this.pageName === page_1.PageName.Connection) {
            this._loadConnectionPage();
        }
        else if (this.pageName === page_1.PageName.DatabaseSummary) {
            this._loadSummaryPage();
        }
        else if (this.pageName === page_1.PageName.TableSummary) {
            this._loadTableSummaryPage();
        }
        else {
            this._loadQueryPage();
        }
    }
    set toolbar(newToolbar) {
        this._toolbar = newToolbar;
        widgets_1.BoxLayout.setStretch(this._toolbar, 0);
        this.layout.insertWidget(0, this._toolbar);
    }
    set page(newPage) {
        const oldPage = this._page;
        if (oldPage !== newPage) {
            this.content.widget = newPage.content;
            this.toolbar = newPage.toolbar;
            this.pageName = newPage.pageName;
            this._page = newPage;
            this._pageChanged.emit(void 0);
            if (oldPage !== null) {
                oldPage.dispose();
            }
            this.content.activate();
        }
    }
    _setConnectionUrl(newConnectionUrl) {
        if (newConnectionUrl !== this._connectionUrl) {
            this._connectionUrl = newConnectionUrl;
            this._connectionUrlChanged.emit(this._connectionUrl);
        }
    }
    _setTableName(newTableName) {
        if (newTableName !== this._tableName) {
            this._tableName = newTableName;
            this._tableNameChanged.emit(this._tableName);
        }
    }
    _setSqlStatement(newStatement) {
        if (newStatement !== this._sqlStatement) {
            this._sqlStatement = newStatement;
            this._sqlStatementChanged.emit(this._sqlStatement);
        }
    }
    _loadConnectionPage() {
        const initialConnectionString = this._connectionUrl;
        const page = new connection_1.ConnectionPage({
            initialConnectionString
        });
        page.connectDatabase.connect((_, connectionUrl) => {
            this._setConnectionUrl(connectionUrl);
            this._loadSummaryPage();
        });
        page.connectionUrlChanged.connect((_, connectionUrl) => {
            this._setConnectionUrl(connectionUrl);
        });
        this.page = page;
    }
    _loadSummaryPage() {
        const connectionUrl = this._connectionUrl;
        const page = new databaseSummary_1.DatabaseSummaryPage({ connectionUrl });
        page.customQueryClicked.connect(() => {
            this._loadQueryPage();
        });
        page.navigateToTable.connect((_, tableName) => {
            this._setTableName(tableName);
            this._loadTableSummaryPage();
        });
        page.navigateBack.connect(() => {
            this._loadConnectionPage();
        });
        this.page = page;
    }
    _loadQueryPage() {
        const options = {
            connectionUrl: this._connectionUrl,
            initialSqlStatement: this._sqlStatement,
            editorFactory: this.editorFactory
        };
        const page = new query_1.QueryPage(options);
        page.backButtonClicked.connect(() => {
            this._loadSummaryPage();
        });
        page.sqlStatementChanged.connect((_, newStatement) => {
            this._setSqlStatement(newStatement);
        });
        this.page = page;
    }
    _loadTableSummaryPage() {
        const tableName = this._tableName;
        const connectionUrl = this._connectionUrl;
        const page = new tableSummary_1.TableSummaryPage({ connectionUrl, tableName });
        page.navigateBack.connect(() => {
            this._loadSummaryPage();
        });
        this.page = page;
    }
    /**
     * Give focus to the content.
     */
    _focusContent() {
        // Focus the content node if we aren't already focused on it or a
        // descendent.
        if (!this.content.node.contains(document.activeElement)) {
            this.content.node.focus();
        }
        // Activate the content asynchronously (which may change the focus).
        this.content.activate();
    }
}
exports.JupyterLabSqlWidget = JupyterLabSqlWidget;
